import re
from dataclasses import dataclass, field

from mayak_api.find_city_id import FindCityID

find_city_id = FindCityID()


@dataclass
class DataGeoObject:
    """
    Служит для создания гео объектов, изначально все свойства = None,
    за исключением представленных ниже \n
        language: list = ['RU'] \n
        status_obj: int = 4 - влияет на то будет ли отображаться объект н сайте или нет\n
        worktime: list  = [] \n
        object_type_gallery: str = 'tourproduct' \n
        awards_gallery: str = '' \n

    """
    base_dir: str
    _object_type_id: int = None
    _title_obj: str = None
    _description: str = None
    _description_full_obj: str = None
    _lat: float = None
    _lon: float = None
    _address: str = None
    _city_id: int = None
    _phone: str = None
    _email: str = None
    _site: str = None
    _instagram: str = None
    _vk: str = None
    _telegram: str = None
    _facebook: str = None

    _object_id: int = None
    _media_gallery_id: int = None

    language: list = field(default_factory=lambda: ['RU'])
    status: int = 4
    worktime: list = field(default_factory=list, repr=True)
    object_type_gallery: str = 'tourproduct'
    awards_gallery: str = ''

    # region Установка гетеров и сетеров
    @property
    def object_type_id(self):
        return self._object_type_id

    @object_type_id.setter
    def object_type_id(self, value):
        if isinstance(value, int):
            self._object_type_id = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def title_obj(self):
        return self._title_obj

    @title_obj.setter
    def title_obj(self, value):
        if isinstance(value, str):
            self._title_obj = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        if isinstance(value, str):
            self._description = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def description_full_obj(self):
        return self._description_full_obj

    @description_full_obj.setter
    def description_full_obj(self, value):
        if isinstance(value, str):
            self._description_full_obj = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def lat(self):
        return self._lat

    @lat.setter
    def lat(self, value):
        if isinstance(value, float):
            self._lat = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def lon(self):
        return self._lon

    @lon.setter
    def lon(self, value):
        if isinstance(value, float):
            self._lon = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def address(self):
        return self._address

    @address.setter
    def address(self, value):
        if isinstance(value, str):
            self._address = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def city_id(self):
        return self._city_id

    @city_id.setter
    def city_id(self, value: int | str):
        """
        Если передаем str, то будет поиск по базе, если int,
        то задается заданное значение
        :param value:
        :return:
        """
        if isinstance(value, int):
            self._city_id = value
        else:
            if not (self.lon or self.lat or self.address):
                raise ValueError("Забыли задать координаты или адрес")

            check_city_id = find_city_id.get_city_id(self)
            if check_city_id:
                self.city_id = check_city_id
            else:
                raise ValueError('city_id не удалось определить')

    @property
    def phone(self):
        return self._phone

    @phone.setter
    def phone(self, value):
        if isinstance(value, str):
            self._phone = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def email(self):
        return self._email

    @email.setter
    def email(self, value):
        if isinstance(value, str):
            self._email = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def site(self):
        return self._site

    @site.setter
    def site(self, value):
        if isinstance(value, str):
            self._site = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def instagram(self):
        return self._instagram

    @instagram.setter
    def instagram(self, value):
        if isinstance(value, str):
            self._instagram = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def vk(self):
        return self._vk

    @vk.setter
    def vk(self, value):
        if isinstance(value, str):
            self._vk = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def telegram(self):
        return self._telegram

    @telegram.setter
    def telegram(self, value):
        if isinstance(value, str):
            self._telegram = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def facebook(self):
        return self._facebook

    @facebook.setter
    def facebook(self, value):
        if isinstance(value, str):
            self._facebook = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def object_id(self):
        return self._object_id

    @object_id.setter
    def object_id(self, value):
        if isinstance(value, int):
            self._object_id = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def media_gallery_id(self):
        return self._media_gallery_id

    @media_gallery_id.setter
    def media_gallery_id(self, value):
        if isinstance(value, int):
            self._media_gallery_id = value
        else:
            raise ValueError("Не корректный формат")

    # endregion

    @staticmethod
    def description_cut(text: str, length: int = 255) -> str:
        """
           Сокращаем текст в пределах длины length до первого переноса или до последней точки
        если ни того ни другого нет, то до последнего пробела
        если пробела тоже нет то просто length символов
        :param text: текст, который не обходимо сократить
        :param length: необходимая длина строки
        :return: возвращает обрезанную строку
        """
        enters = [m.start() for m in re.finditer('\n', text[:length])]
        if enters:
            return text[:enters[0]]
        points = [m.start() for m in re.finditer('\.', text[:length])]
        if points:
            return text[:points[-1] + 1]
        spaces = [m.start() for m in re.finditer(' ', text[:length])]
        if spaces:
            if spaces[-1] == 0:
                return ' '
            else:
                return text[:spaces[-1]]
        return text[:length]

    def create_correct_dict(self):
        dict_for_update = {k.lstrip('_'): v for k, v in self.__dict__.items()}
        return dict_for_update
