import re
from dataclasses import dataclass
from datetime import datetime


@dataclass
class DataReview:
    """
        Служит для работы с отзывами, изначально все свойства = None,
    за исключением представленных ниже \n
        status_rew: int = 4 - влияет на то будет ли отображаться объект н сайте или нет\n

    14 полей явлются обязательными
         необязательные: сustomer_id, external_link, reviewer_email, create_date_time

    """
    _type: int = None
    _uid: int = None
    _platform_id: int = None
    _review_platform_id: str = None
    _title: str = None
    _description: str = None
    _review_date_time: datetime = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    _external_rating: str = None
    _rating: int = None
    _like_count: int = None
    _dislike_count: int = None
    _additional_data: dict = None
    _media_data: dict = None
    _status: int = 1

    _customer_id: int = None
    _external_link: str = None
    _reviewer_email: str = None
    _create_date_time: datetime = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    _review_id: int = None

    # region Установка гетеров и сетеров
    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, value: int):
        if isinstance(value, int):
            self._type = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def uid(self):
        return self._uid

    @uid.setter
    def uid(self, value: int):
        if isinstance(value, int):
            self._uid = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def platform_id(self):
        return self._platform_id

    @platform_id.setter
    def platform_id(self, value: int):
        if isinstance(value, int):
            self._platform_id = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def review_platform_id(self):
        return self._review_platform_id

    @review_platform_id.setter
    def review_platform_id(self, value: str):
        if isinstance(value, str):
            self._review_platform_id = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value: str):
        if isinstance(value, str):
            self._title = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value: str):
        if isinstance(value, str):
            regrex = re.compile(pattern="["
                                        u"\U00000000-\U00000009"
                                        u"\U0000000B-\U0000001F"
                                        u"\U00000080-\U00000400"
                                        u"\U00000402-\U0000040F"
                                        u"\U00000450-\U00000450"
                                        u"\U00000452-\U0010FFFF"
                                        "]+", flags=re.UNICODE)

            value = regrex.sub(r'', value)

            self._description = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def review_date_time(self):
        return self._review_date_time

    @review_date_time.setter
    def review_date_time(self, value: datetime):
        if isinstance(value, datetime):
            self._review_date_time = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def external_rating(self):
        return self._external_rating

    @external_rating.setter
    def external_rating(self, value: str):
        if isinstance(value, str):
            self._external_rating = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def rating(self):
        return self._rating

    @rating.setter
    def rating(self, value: int):
        if isinstance(value, int):
            self._rating = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def like_count(self):
        return self._like_count

    @like_count.setter
    def like_count(self, value: int):
        if isinstance(value, int):
            self._like_count = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def dislike_count(self):
        return self._dislike_count

    @dislike_count.setter
    def dislike_count(self, value: int):
        if isinstance(value, int):
            self._dislike_count = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def additional_data(self):
        return self._additional_data

    @additional_data.setter
    def additional_data(self, value: dict):
        if isinstance(value, dict):
            self._additional_data = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def media_data(self):
        return self._media_data

    @media_data.setter
    def media_data(self, value: dict):
        if isinstance(value, dict):
            self._media_data = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def status_rew(self):
        return self._status

    @status_rew.setter
    def status_rew(self, value: int):
        if isinstance(value, int):
            self._status = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def customer_id(self):
        return self._customer_id

    @customer_id.setter
    def customer_id(self, value: int):
        if isinstance(value, int):
            self._customer_id = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def external_link(self):
        return self._external_link

    @external_link.setter
    def external_link(self, value: str):
        if isinstance(value, str):
            self._external_link = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def reviewer_email(self):
        return self._reviewer_email

    @reviewer_email.setter
    def reviewer_email(self, value: str):
        if isinstance(value, str):
            self._reviewer_email = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def create_date_time(self):
        return self._create_date_time

    @create_date_time.setter
    def create_date_time(self, value: datetime):
        if isinstance(value, datetime):
            self._create_date_time = value
        else:
            raise ValueError("Не корректный формат")

    @property
    def review_id(self):
        return self._review_id

    @review_id.setter
    def review_id(self, value: datetime):
        if isinstance(value, int):
            self._review_id = value
        else:
            raise ValueError("Не корректный формат")

    # endregion

    def create_correct_dict(self):
        dict_for_update = {k.lstrip('_'): v for k, v in self.__dict__.items()}
        return dict_for_update
