import re
from dadata import Dadata
from mayak_api.sql_processor import SQLProcessor


class FindCityID(SQLProcessor):

    def __init__(self):
        self.config_dict = self.config('mayak_api', 'config.ini')

    @staticmethod
    def searching_match_between_address_and_get_data(city_of_address: list,
                                                     list_city_id_of_db: list[list]) -> int | bool:
        """
        Метод ищет соответствия между адресом (self.address) и списком городов полученным из БД
        и при совпадении возвращает id города, в противном случае False
        :param city_of_address: список городов из self.address
        :param list_city_id_of_db: список городов из БД с city_id
        :return: city_id:int или False
        """
        for part_address in city_of_address:
            for sublist in list_city_id_of_db:
                if sublist[1] == part_address:
                    city_id: int = sublist[0]
                    return city_id

                result = re.sub(r'^.*\b([А-ЯA-Z][а-яa-z]*)\b', r'\1',
                                part_address).strip()
                print(f'{result=}')

                if sublist[1] == result:
                    city_id: int = sublist[0]
                    return city_id

                result_1 = re.sub(r'г\.* ', '',
                                  part_address).strip()
                print(f'{result_1=}')

                if sublist[1] == result_1:
                    city_id: int = sublist[0]
                    return city_id
        return False

    def _find_city_id(self,
                      difference: float,
                      city_of_address_begin: list,
                      get_dict_data: dict,
                      ) -> int | bool:
        """
        Метод по поиску id с использованием SQL запроса возвращает city_id,
        в случае ошибки/пустого значения - False

        :param difference: разница от основной координаты
        :param base_dir: директория, где находbится скрипт
                    (os.path.dirname(os.path.abspath(__file__)))
        :param city_of_address_begin: список, полученный из адреса объекта
        :param get_dict_data: словарь с данными
        :return: возвращает city_id или False
        """
        base_dir = get_dict_data['base_dir']
        lon_min = get_dict_data['lon'] - difference
        lon_max = get_dict_data['lon'] + difference
        lat_min = get_dict_data['lat'] - difference
        lat_max = get_dict_data['lat'] + difference

        df_psql = self.get_data_of_db_psql_or_mysql(
            self.config_dict,
            'select_mysql_begin_data_city.sql',
            base_dir,
            another_db='debezium_db',
            type_db='mysql',
            params_names=(
                get_dict_data['lon'],
                get_dict_data['lat'],
                lon_min,
                lon_max,
                lat_min,
                lat_max,
                get_dict_data['lon'],
                get_dict_data['lat'],
            )
        )

        word_exception = ['край', 'Край', 'обл ', 'респ',
                          'Респ', 'ул ', 'пр-кт', 'пл ',
                          'Российская Федерация', 'ул.', 'пл.',
                          'обл.']

        city_of_address_after_filter = [item.strip() for item in city_of_address_begin
                                        if all(j not in item.strip() for j in word_exception)]

        if len(city_of_address_after_filter) == 0:
            city_of_address_after_filter = city_of_address_begin

        print(f'{city_of_address_after_filter=}')

        list_city_id = df_psql.values.tolist()

        if len(list_city_id) != 0:
            print(1)
            print(f'{list_city_id=}')
            city_id_values: int = self.searching_match_between_address_and_get_data(city_of_address_after_filter,
                                                                                    list_city_id)

            if city_id_values:
                return city_id_values

        for part_address in city_of_address_after_filter:
            print(2)
            print(f'{part_address=}')

            result = re.sub(r'^.*\b([А-ЯA-Z][а-яa-z]*)\b', r'\1',
                            part_address).strip()

            result_1 = re.sub(r'г\.* ', '',
                              part_address).strip()

            result_dadata = self._get_city_with_help_dadata(get_dict_data['lat'], get_dict_data['lon'])
            result_2 = result_dadata[0]['data']['city']

            for i in [result, result_1, result_2]:
                city_id = self._get_city_id_with_only_city(i, base_dir)
                if city_id:
                    return city_id

        print('Ничего')
        return False

    def _get_city_with_help_dadata(self, lat: float, lon: float, count: int = 1) -> list[dict]:
        """
        Получение c помощью сервиса dadata списка адресов по координатам
        :param lat: широта
        :param lon: долгота
        :param count: число адресов, которые вернет сервис, по умолчанию = 1
        :return: возвращает список словарей с данными
        """

        dadata = Dadata(self.config_dict['dadata_api_key'])
        result_dadata: list[dict] = dadata.geolocate('address', lat,
                                                     lon, count=count)
        return result_dadata

    def _get_city_id_with_only_city(self, city_name, base_dir) -> int | bool:
        print(city_name)
        df_psql = self.get_data_of_db_psql_or_mysql(
            self.config_dict,
            'select_mysql_get_city_id.sql',
            base_dir,
            another_db='debezium_db',
            type_db='mysql',
            params_names=(
                city_name
            )
        )

        if not df_psql.empty:
            print(df_psql.values.tolist())
            city_id: int = df_psql.values.tolist()[0][0]
            return city_id
        else:
            return False

    def get_city_id(self, data) -> int | bool:
        """
        Основная логика поиска city_id
        :param data: объекта класса Data, который обязательно должен содержать следующие свойства \n
                lat: float
                lon: float
                address: str
        :return:
        """
        get_dict_data: dict = data.__dict__

        find_city_begin = get_dict_data['address'].split(',')

        city_id: int | bool = self._find_city_id(0.04, find_city_begin, get_dict_data)

        if not city_id:
            print('Не вышло')
            return False
        else:
            return city_id


